/*
 *  Copyright (C) 2008  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include "iksemel_extra.h"
#include "xmpp.h"
#include "sasl.h"
#include "bind.h"
#include "disco.h"
#include "xmpp_internal.h"

void
hc_xmpp_hook_bind (hc_xmpp_t *xmpp, iks *stanza)
{
  hc_xmpp_bind_result (xmpp, stanza);
  if (hc_xmpp_status (xmpp) == HC_XMPP_BOUND &&
      hc_xmpp_is_session_supported (xmpp))
    hc_xmpp_session (xmpp);
}

void
hc_xmpp_hook_session (hc_xmpp_t *xmpp, iks *stanza)
{
  hc_xmpp_session_result (xmpp, stanza);
  if (hc_xmpp_status (xmpp) == HC_XMPP_SESSION)
    hc_xmpp_send_disco_info (xmpp, hc_xmpp_server (xmpp));
}

void
hc_xmpp_hook_disco (hc_xmpp_t *xmpp, iks *stanza)
{
  hc_xmpp_recv_disco (xmpp, stanza);
}

int
hc_xmpp_hook (void *data, int type, iks *stanza)
{
  hc_xmpp_t *xmpp = (hc_xmpp_t *) data;
  if (!iks_strcmp (iks_name (stanza), "message"))
    {
      hc_xmpp_recv_message (xmpp, stanza);
    }
  else if (!iks_strcmp (iks_name (stanza), "presence"))
    {
      hc_xmpp_recv_presence (xmpp, stanza);
    }
  else if (!iks_strcmp (iks_name (stanza), "iq"))
    {
      char *ns = iks_find_attrib (iks_child (stanza), "xmlns");
      hc_xmpp_hook_t hook;
      if (g_hash_table_lookup_extended (xmpp->nshooks, ns,
                                        NULL, (gpointer *) &hook))
        hook (xmpp, stanza);
    }
  else if (!iks_strcmp (iks_name (stanza), "stream:features"))
    {
      hc_xmpp_features (data, stanza);
      if (hc_xmpp_is_sasl_supported (data) & !hc_xmpp_is_sasl_enabled (data))
        {
          hc_xmpp_sasl_authenticate (data);
        }
      if (hc_xmpp_is_bind_supported (data))
        {
          hc_xmpp_bind (data);
        }
    }
  else if (!iks_strcmp (iks_find_attrib (stanza, "xmlns"), HC_XMPP_NS_SASL))
    {
      hc_xmpp_sasl_iterate (data, stanza);
      if (hc_xmpp_status (data) == HC_XMPP_AUTHENTICATED)
        {
          hc_xmpp_send_stream (data);
        }
    }
  return IKS_OK;
}

void
hc_xmpp_send_stream (hc_xmpp_t *xmpp)
{
  char *buffer = NULL;
  asprintf (&buffer, "<stream:stream xmlns='jabber:client' "
                     "xmlns:stream='http://etherx.jabber.org/streams' "
                     "version='1.0' to='%s'>", hc_xmpp_server (xmpp));
  hc_xmpp_send_buffer (xmpp, buffer, 0);
  free (buffer);
}
