/*
 *  Copyright (C) 2008  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <gtk/gtk.h>
#include <stdio.h>
#include <glib.h>
#include <udns.h>
#include "tcp_connect.h"
#include "xmpp.h"

static gboolean
rploop (GIOChannel *src, GIOCondition cond, gpointer xmpp)
{
  hc_xmpp_read_and_parse (xmpp);
  return TRUE;
}

GtkWidget *entry1;
GtkWidget *entry2;
GtkWidget *entry3;

extern void hc_xmpp_chat_open (hc_xmpp_t *);

static void
connect_clicked (GtkWidget *button, gpointer data)
{
  GIOChannel *channel;
  hc_xmpp_t *xmpp;
  char *server;
  char *user;
  char *password;
  server = (char *) gtk_entry_get_text (GTK_ENTRY (entry1));
  user = (char *) gtk_entry_get_text (GTK_ENTRY (entry2));
  password = (char *) gtk_entry_get_text (GTK_ENTRY (entry3));
  xmpp = hc_xmpp_new (hc_xmpp_hook, server, user, password);
  hc_xmpp_send_stream (xmpp);
  channel = g_io_channel_unix_new (hc_xmpp_fd (xmpp));
  g_io_add_watch (channel, G_IO_IN, rploop, xmpp);
  hc_xmpp_chat_open (xmpp);
}

static void
ui (void)
{
  GtkWidget *window;
  GtkWidget *vbox;
  GtkWidget *button;
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  vbox = gtk_vbox_new (TRUE, 5);
  entry1 = gtk_entry_new ();
  entry2 = gtk_entry_new ();
  entry3 = gtk_entry_new ();
  gtk_entry_set_visibility (GTK_ENTRY (entry3), FALSE);
  button = gtk_button_new_with_label ("Connect");
  g_signal_connect (G_OBJECT (button), "clicked",
                    G_CALLBACK (connect_clicked), NULL);
  gtk_container_add (GTK_CONTAINER (window), vbox);
  gtk_container_add (GTK_CONTAINER (vbox), entry1);
  gtk_container_add (GTK_CONTAINER (vbox), entry2);
  gtk_container_add (GTK_CONTAINER (vbox), entry3);
  gtk_container_add (GTK_CONTAINER (vbox), button);
  gtk_widget_show_all (window);
}

int
main (int argc, char **argv)
{
  GMainLoop *loop;
  gtk_init (&argc, &argv);
  dns_init (NULL, 1);
  loop = g_main_loop_new (g_main_context_default (), TRUE);
  ui ();
  g_main_loop_run (loop);
  return 0;
}
